import { Assert } from '../../../Diagnostics/Assert.js';
import { EventNode } from '../../../Nodes/EventNode.js';
import { NodeDescription } from '../../../Nodes/Registry/NodeDescription.js';
import { Socket } from '../../../Sockets/Socket.js';
export class LifecycleOnStart extends EventNode {
    constructor(description, graph) {
        super(description, graph, [], [new Socket('flow', 'flow')]);
        this.onStartEvent = undefined;
    }
    init(engine) {
        Assert.mustBeTrue(this.onStartEvent === undefined);
        this.onStartEvent = () => {
            engine.commitToNewFiber(this, 'flow');
        };
        const lifecycleEvents = engine.graph.registry.abstractions.get('ILifecycleEventEmitter');
        lifecycleEvents.startEvent.addListener(this.onStartEvent);
    }
    dispose(engine) {
        Assert.mustBeTrue(this.onStartEvent !== undefined);
        if (this.onStartEvent !== undefined) {
            const lifecycleEvents = engine.graph.registry.abstractions.get('ILifecycleEventEmitter');
            lifecycleEvents.startEvent.removeListener(this.onStartEvent);
        }
    }
}
LifecycleOnStart.Description = new NodeDescription('lifecycle/onStart', 'Event', 'On Start', (description, graph) => new LifecycleOnStart(description, graph));
//# sourceMappingURL=LifecycleOnStart.js.map