import { Assert } from '../../../Diagnostics/Assert.js';
import { EventNode } from '../../../Nodes/EventNode.js';
import { NodeDescription } from '../../../Nodes/Registry/NodeDescription.js';
import { Socket } from '../../../Sockets/Socket.js';
export class LifecycleOnTick extends EventNode {
    constructor(description, graph) {
        super(description, graph, [], [
            new Socket('flow', 'flow'),
            new Socket('float', 'deltaSeconds'),
            new Socket('float', 'time')
        ]);
        this.onTickEvent = undefined;
    }
    init(engine) {
        Assert.mustBeTrue(this.onTickEvent === undefined);
        let lastTickTime = Date.now();
        this.onTickEvent = () => {
            const currentTime = Date.now();
            const deltaSeconds = (currentTime - lastTickTime) * 0.001;
            this.writeOutput('deltaSeconds', deltaSeconds);
            this.writeOutput('time', Date.now());
            engine.commitToNewFiber(this, 'flow');
            lastTickTime = currentTime;
        };
        const lifecycleEvents = engine.graph.registry.abstractions.get('ILifecycleEventEmitter');
        lifecycleEvents.tickEvent.addListener(this.onTickEvent);
    }
    dispose(engine) {
        Assert.mustBeTrue(this.onTickEvent !== undefined);
        if (this.onTickEvent !== undefined) {
            const lifecycleEvents = engine.graph.registry.abstractions.get('ILifecycleEventEmitter');
            lifecycleEvents.tickEvent.removeListener(this.onTickEvent);
        }
    }
}
LifecycleOnTick.Description = new NodeDescription('lifecycle/onTick', 'Event', 'On Tick', (description, graph) => new LifecycleOnTick(description, graph));
//# sourceMappingURL=LifecycleOnTick.js.map